# Eiko Fried, Network Analysis Workshop Universidad Complutense de Madrid, September 19-20 2017
# More information: www.eiko-fried.com, Twitter @EikoFried, eikofried@gmail.com



############### Time series: network estimation and inference in N=1

# ---------------------------------------------------------------------------------------
# ---------- 1. Load libaries -----------------------------------------------------------
# ---------------------------------------------------------------------------------------

library("qgraph")
library("graphicalVAR")
library("mlVAR")

# ---------------------------------------------------------------------------------------
# ---------- 2. Data preparation --------------------------------------------------------
# ---------------------------------------------------------------------------------------

# set working directly to where the data file is, then:
Data1 <- read.csv("Patient1.csv")
View(Data1)

# define variables
Vars <- c("relaxed", "sad", "nervous",
          "concentration", "tired",
          "rumination", "bodily.discomfort")
          
# Encode time variable in a way R understands:
Data1$time <- as.POSIXct(Data1$time)

# Extract days:
Data1$Day <- as.Date(Data1$time)

# ---------------------------------------------------------------------------------------
# ---------- 3. Estimate n=1 network ----------------------------------------------------
# ---------------------------------------------------------------------------------------

# Estimate a graphical VAR model on the dataset using EBIC tuning parameter γ = 0
# use arguments vars = Vars to set the variables, and dayvar = "Day" to remove nights
Results <- graphicalVAR(
  Data1,
  vars = Vars,
  dayvar = "Day",
  gamma = 0)

plot(Results)
dev.off()

Results1 <- graphicalVAR(
  Data1,
  vars = Vars,
  dayvar = "Day",
  gamma = 0.5)   #0.5 is the default

plot(Results1)
dev.off()

# Detrend variables
lm_tired <- lm(tired ~ time, data = Data1)
plot(Data1$time,Data1$tired, type="b", ylim=c(0,7))
abline(lm_tired, col="red")

summary(lm_tired)

# This effect is significant; we can now “detrend” tired as follows:
Data1$tired[!is.na(Data1$tired)] <- residuals(lm_tired)
lm_tired1 <- lm(tired ~ time, data = Data1)
plot(Data1$time,Data1$tired, type="b", ylim=c(-7,7))
abline(lm_tired1, col="red")


# Loop to detrend everything if necessary
for (v in seq_along(Vars)){
  lmResult <- lm(Data1[,Vars[v]] ~ Data1$time)
  if (anova(lmResult)[["Pr(>F)"]][1] < 0.05){
    Data1[,Vars[v]][!is.na(Data1[,Vars[v]])] <- residuals(lmResult)
  }
}

Results2 <- graphicalVAR(
  Data1,
  vars = Vars,
  dayvar = "Day",
  gamma = 0,
  verbose=FALSE
)

# New detrended results
plot(Results2, details=T)
dev.off()

# Can also plot only single results, e.g.:
plot(Results2, "PCC", layout = "circle")

# Output saved in:
Results2$PCC  # contemporaneous
Results2$PDC  # directed





############### Time series: network estimation and inference in N>1

# ---------------------------------------------------------------------------------------
# ---------- 4. Data preparation --------------------------------------------------------
# ---------------------------------------------------------------------------------------

Data2 <- read.table(file="BringmannData.csv",header=TRUE,sep=",")
View(Data2)

vars <- c("cheerful","pleasant","worry","fearful","sad","relaxed")
idvar <- "id"
beepvar <- "beep"
dayvar <- "day"

# ---------------------------------------------------------------------------------------
# ---------- 5. Estimate n>1 model ------------------------------------------------------
# ---------------------------------------------------------------------------------------

# Plot the average (fixed effects) temporal network, using a circular layout and hiding non-significant edges
res <- mlVAR(Data2, vars, idvar, dayvar, beepvar, lags = 1,
             temporal = "correlated", contemporaneous = "correlated")

plot(res, "temporal", layout = "circle", nonsig = "hide")

# Plot average (fixed effects) contemporaneous partial correlation network, use circle layout and hide non-significant edges
# Use "and" rule to minimize type-1 error; compare the network to the temporal network
# Also estimate between subjects effects

pdf("eiko.pdf", width=8, height=5)
layout(t(1:3))
plot(res, "contemporaneous", layout = "circle", nonsig = "hide", rule = "and", title="Contemporaneous", details=T)
plot(res, "temporal", layout = "circle", nonsig = "hide", title="Temporal", details=T)
plot(res, "between", layout = "circle", nonsig = "hide", rule = "and", title="Between-subjects", details=T)
dev.off()
# Between-subjects network shows less connections. This is network between means -> less observations making it harder to reach statistical significance


# Future: time-varying models. See work by Laura Bringmann & Jonas Haslbeck. 